@@ -1,15 +1,23 @@
-PlotAno <- function(exp_ano, obs_ano = NULL, sdates, 
-                    toptitle = c('', '', '', '', '', '', '', '', '', '', '', '',
-                                 '', '', ''),
-                    ytitle = c('', '', '', '', '', '', '', '', '', '', '', '', 
-                               '', '', ''), 
-                    limits = NULL, legends = NULL, freq = 12, biglab = FALSE, 
-                    fill = TRUE, memb = TRUE, ensmean = TRUE, linezero = FALSE, 
-                    points = FALSE, vlines = NULL, 
-                    fileout = c('output1_plotano.eps', 
-                    'output2_plotano.eps', 'output3_plotano.eps', 
-                    'output4_plotano.eps', 'output5_plotano.eps'), 
-                    sizetit = 1) {           
+PlotAno <- function(exp_ano, obs_ano = NULL, sdates, toptitle = rep('', 15),
+                    ytitle = rep('', 15), limits = NULL, legends = NULL, 
+                    freq = 12, biglab = FALSE, fill = TRUE, memb = TRUE, 
+                    ensmean = TRUE, linezero = FALSE, points = FALSE, 
+                    vlines = NULL, sizetit = 1, 
+                    fileout = paste0('output', 1:5, '_plotano.eps'), 
+                    width = 8, height = 5, size_units = 'in', res = 100, ...) {
+  # Process the user graphical parameters that may be passed in the call
+  ## Graphical parameters to exclude
+  excludedArgs <- c("cex", "cex.axis", "cex.lab", "cex.main", "col", "fin", "lab", "las", "lty", "lwd", "mai", "mgp", "new", "pch", "pin", "ps", "pty")
+  userArgs <- .FilterUserGraphicArgs(excludedArgs, ...)
+
+  # If there is any filenames to store the graphics, process them
+  # to select the right device 
+  if (!is.null(fileout)) {
+    deviceInfo <- .SelectDevice(fileout = fileout, width = width, height = height, units = size_units, res = res)
+    saveToFile <- deviceInfo$fun
+    fileout <- deviceInfo$files
+  }
+
   #
   #  Get some arguments
   # ~~~~~~~~~~~~~~~~~~~~
@@ -87,14 +95,25 @@ PlotAno <- function(exp_ano, obs_ano = NULL, sdates,
     #  Define plot layout
     # ~~~~~~~~~~~~~~~~~~~~
     #
-    postscript(fileout[jexp], width = 550, height = 300)
+
+    # Open connection to graphical device
+    if (!is.null(fileout)) {
+      saveToFile(fileout[jexp])
+    } else if (names(dev.cur()) == 'null device') {
+      dev.new(units = size_units, res = res, width = width, height = height)
+    }
+
+
+    # Load the user parameters
+    par(userArgs)
+
     if (biglab) {
       par(mai = c(1, 1.1, 0.5, 0), mgp = c(2.8, 0.9, 0))
       par(cex = 1.3, cex.lab = 2, cex.axis = 1.8)
       cexmain <- 2.2
       legsize <- 1.5
     } else {
-      par(mai = c(0.8, 0.8, 0.5, 0.1), mgp = c(2, 0.5, 0))
+      par(mai = c(0.8, 0.8, 0.5, 0.3), mgp = c(2, 0.5, 0))
       par(cex = 1.3, cex.lab = 1.5, cex.axis = 1.1)
       cexmain <- 1.5
       legsize <- 1
@@ -194,13 +213,15 @@ PlotAno <- function(exp_ano, obs_ano = NULL, sdates,
     }
     if (is.null(legends) == FALSE) {
       if (points) {
-        legend(0, ul, legends[1:nobs], lty = 3, lwd = 10, col = 1, 
+        legend('topleft', legends[1:nobs], lty = 3, lwd = 10, col = 1, 
                cex = legsize)
       } else {
-        legend(0, ul, legends[1:nobs], lty = type[1:nobs], 
+        legend('topleft', ul, legends[1:nobs], lty = type[1:nobs], 
                lwd = thickness[1:nobs], col = 1, cex = legsize)
       }
     }
-    dev.off()
+
+    # If the graphic was saved to file, close the connection with the device
+    if(!is.null(fileout)) dev.off()
   }
 }
